package com.nisum.offertransactionservice.config;

import feign.RetryableException;
import feign.Retryer;

public class CustomRetryer implements Retryer {

    private final Integer maxAttempts;
    private final Long backoff;
    int attempt;

    public CustomRetryer(Long backoff, Integer maxAttempts) {
        this.backoff = backoff;
        this.maxAttempts = maxAttempts;
        this.attempt = 1;
    }

    public void continueOrPropagate(RetryableException e) {
        if (attempt++ >= maxAttempts) {
            throw e;
        }

        try {
            Thread.sleep(backoff);
        } catch (InterruptedException ignored) {
            Thread.currentThread().interrupt();
        }
    }

    @Override
    public Retryer clone() {
        return new CustomRetryer(backoff, maxAttempts);
    }
}
