package com.nisum.offertransactionservice.service;

import com.nisum.offertransactionservice.dao.EndOfSaleRepo;
import com.nisum.offertransactionservice.dto.EndOfSaleReq;
import com.nisum.offertransactionservice.dto.Item;
import com.nisum.offertransactionservice.dto.OfferTransactionResponse;
import com.nisum.offertransactionservice.model.OfferLookup;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.*;

public class EndOfSaleTest {

    @InjectMocks
    private EndOfSaleService endOfSaleService;

    @Mock
    private EndOfSaleRepo endOfSaleRepo;

    private static final String expectedUUID="123456788899999998";


    @BeforeEach
    public  void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }


    @DisplayName("This test will be test the  putEndofSaleResInDb method ")
    @ParameterizedTest
    @MethodSource("getEndOfSaleReqSource")
    public void putEndofSaleResInDbTest( EndOfSaleReq endOfSaleReq) throws IOException {
        Mockito.when(endOfSaleRepo.save(ArgumentMatchers.any())).thenReturn(ArgumentMatchers.any());
        String uuid = endOfSaleService.putEndofSaleResInDb(endOfSaleReq);
        assertAll(
                "uuid test",
                () -> assertEquals(expectedUUID,uuid,()->"uuid should match to the expected"),
                () -> assertNotNull(uuid,()->"uuid should not be returned as null")
        );
    }

    @ParameterizedTest
    @MethodSource("getEndOfSaleReqSource")
    public void putEndofSaleResInDbNegTest(EndOfSaleReq endOfSaleReq) throws IOException {
        Mockito.when(endOfSaleRepo.save(ArgumentMatchers.any())).thenReturn(ArgumentMatchers.any());
        String uuid = endOfSaleService.putEndofSaleResInDb(endOfSaleReq);
        assertAll(
                "uuid test",
                () -> assertNotNull(uuid,()->"uuid should not be returned as null")
        );
    }

    private static  EndOfSaleReq getEndOfSaleReq() {
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq();
        endOfSaleReq.setOfferTransactionResponse(retOfferTransRes());
        endOfSaleReq.setOffers(returnOffers());

        return endOfSaleReq;
    }

    private static Stream<Arguments> getEndOfSaleReqSource() {
        return Stream.of(
                Arguments.of(getEndOfSaleReq())
        );
    }


    private static OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        offerTransactionResponse.setTransactionId("123456788899999998");
        return offerTransactionResponse;
    }

    private static OfferTransactionResponse retOfferTransNegRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        //offerTransactionResponse.setTransactionId("123456788899999998");
        return offerTransactionResponse;
    }

    public static List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup("1", "12", 23L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("2", "13", 24L, "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }

}
